
from django.test import TestCase
import ajax_select


class TestRegistry(TestCase):

    def test_lookup_py_is_autoloaded(self):
        """Django >= 1.7 autoloads tests/lookups.py"""
        is_registered = ajax_select.registry.is_registered('person')
        self.assertTrue(is_registered)

    def test_back_compatible_loads_by_settings(self):
        """a module and class specified in settings"""
        self.assertTrue(ajax_select.registry.is_registered('book'))

    def test_autoconstruct_from_spec(self):
        """a dict in settings specifying model and lookup fields"""
        self.assertTrue(ajax_select.registry.is_registered('author'))

    def test_unsetting_a_channel(self):
        """settings can unset a channel that was specified in a lookups.py"""
        # self.assertFalse(ajax_select.registry.is_registered('user'))
        self.assertFalse(
            ajax_select.registry.is_registered('was-never-a-channel'))

    # def test_reimporting_lookup(self):
    #     """
    #     Importing a lookup should not re-register it after app launch is completed.
    #     """
    #     # importing this file will cause the @register to be called
    #     from tests import lookups  # noqa
    #     # should not have over-ridden what is in settings
    #     # registry is already frozen
    #     self.assertFalse(ajax_select.registry.is_registered('user'))




from django.test import TestCase
from django.contrib.auth.models import User
from django.test import Client


class TestViews(TestCase):

    def setUp(self):
        self.user = User.objects.create_superuser(username='admin',
            email='email@example.com',
            password='password')
        self.client = Client()
        self.client.login(username='admin', password='password')



"""
Testing the register and autoloading.

Should not be used by other tests.
"""
from django.contrib.auth.models import User
from django.utils.html import escape

import ajax_select

from tests.models import Author, Person, PersonWithTitle


@ajax_select.register('person')
class PersonLookup(ajax_select.LookupChannel):

    model = Person

    def get_query(self, q, request):
        return self.model.objects.filter(name__icontains=q)

    def get_result(self, obj):
        return obj.name

    def format_match(self, obj):
        return "%s<div><i>%s</i></div>" % (escape(obj.name), escape(obj.email))

    def format_item_display(self, obj):
        return "%s<div><i>%s</i></div>" % (escape(obj.name), escape(obj.email))


@ajax_select.register('person-with-title')
class PersonWithTitleLookup(ajax_select.LookupChannel):

    model = PersonWithTitle

    def get_query(self, q, request):
        return self.model.objects.filter(title__icontains=q)

    def get_result(self, obj):
        return "{} {}".format(obj.name, obj.title)


@ajax_select.register('user')
class UserLookup(ajax_select.LookupChannel):

    """
    Test if you can unset a lookup provided by a third-party application.
    In this case it exposes User without any auth checking
    and somebody could manually check the ajax URL and find out
    if a user email exists.
    So you might want to turn this channel off
    by settings.AJAX_LOOKUP_CHANNELS['user'] = None
    """

    model = User

    def get_query(self, q, request):
        return self.model.objects.filter(email=q)


@ajax_select.register('name')
class NameLookup(ajax_select.LookupChannel):

    def get_query(self, q, request):
        return ['Joseph Simmons', 'Darryl McDaniels', 'Jam Master Jay']


@ajax_select.register('author')
class AuthorLookup(ajax_select.LookupChannel):

    model = Author



from django.test import TestCase
from ajax_select import fields
from tests.models import Book


class TestAutoCompleteSelectWidget(TestCase):

    def test_render(self):
        channel = 'book'
        widget = fields.AutoCompleteSelectWidget(channel)
        out = widget.render('book', None)
        self.assertTrue('autocompleteselect' in out)

    def test_render_with_value(self):
        channel = 'book'
        widget = fields.AutoCompleteSelectWidget(channel)
        book = Book.objects.create(name='book')
        out = widget.render('book', book.pk)
        self.assertTrue('autocompleteselect' in out)

    def test_render_required_field(self):
        field = fields.AutoCompleteSelectField('book', required=True)
        widget = field.widget

        book = Book.objects.create(name='book')
        out = widget.render('book', book.pk)
        self.assertTrue('autocompleteselect' in out)
        self.assertTrue('required' not in out)


class TestAutoCompleteSelectMultipleWidget(TestCase):

    def test_render(self):
        channel = 'book'
        widget = fields.AutoCompleteSelectMultipleWidget(channel)
        out = widget.render('book', None)
        self.assertTrue('autocompleteselectmultiple' in out)

    def test_render_with_query_set(self):
        channel = 'book'
        widget = fields.AutoCompleteSelectMultipleWidget(channel)
        Book.objects.create(name='book')
        out = widget.render('book', Book.objects.all())
        self.assertTrue('autocompleteselectmultiple' in out)


class TestAutoCompleteWidget(TestCase):

    def test_render(self):
        channel = 'book'
        widget = fields.AutoCompleteWidget(channel)
        out = widget.render('book', None)
        self.assertTrue('autocomplete' in out)


class TestAutoCompleteSelectField(TestCase):

    def test_has_changed(self):
        field = fields.AutoCompleteSelectField('book')
        self.assertFalse(field.has_changed(1, '1'))
        self.assertFalse(field.has_changed('abc', 'abc'))
        self.assertTrue(field.has_changed(1, '2'))
        self.assertFalse(field.has_changed(None, ''))
        self.assertFalse(field.has_changed(None, None))
        self.assertFalse(field.has_changed('', None))


class TestAutoCompleteSelectMultipleField(TestCase):

    def test_has_changed(self):
        field = fields.AutoCompleteSelectMultipleField('book')
        self.assertFalse(field.has_changed([1], ['1']))
        self.assertFalse(field.has_changed(['abc'], ['abc']))
        self.assertTrue(field.has_changed([1], ['2']))

    def test_has_changed_blank_input(self):
        field = fields.AutoCompleteSelectMultipleField('book')
        self.assertTrue(field.has_changed(None, ['1']))
        self.assertFalse(field.has_changed(None, []))



from django.apps import AppConfig


class AjaxSelectConfig(AppConfig):

    """
    Django 1.7+ enables initializing installed applications
    and autodiscovering modules.

    On startup, search for and import any modules called `lookups.py` in all installed apps.
    Your LookupClass subclass may register itself.
    """

    name = 'ajax_select'
    verbose_name = 'Ajax Selects'

    def ready(self):
        from ajax_select.registry import registry
        registry.load_channels()



from django.core.exceptions import PermissionDenied
from django.utils.encoding import force_text
from django.utils.html import escape


class LookupChannel(object):

    """
    Subclass this, setting the model and implementing methods to taste.

    Attributes::

        model (Model): The Django Model that this lookup channel will search for.
        plugin_options (dict): Options passed to jQuery UI plugin that are specific to this channel.
        min_length (int): Minimum number of characters user types before a search is initiated.

            This is passed to the jQuery plugin_options.
            It is used in jQuery's UI when filtering results from its own cache.

            It is also used in the django view to prevent expensive database queries.
            Large datasets can choke if they search too often with small queries.
            Better to demand at least 2 or 3 characters.
    """

    model = None
    plugin_options = {}
    min_length = 1

    def get_query(self, q, request):
        """
        Return a QuerySet searching for the query string `q`.

        Note that you may return any iterable so you can return a list or even
        use yield and turn this method into a generator.

        Args:
            q (str, unicode): The query string to search for.
            request (Request): This can be used to customize the search by User
                or to use additional GET variables.

        Returns:
            (QuerySet, list, generator): iterable of related_models
        """
        kwargs = {"%s__icontains" % self.search_field: q}
        return self.model.objects.filter(**kwargs).order_by(self.search_field)

    def get_result(self, obj):
        """
        The text result of autocompleting the entered query.

        For a partial string that the user typed in, each matched result is
        here converted to the fully completed text.

        This is currently displayed only for a moment in the text field after
        the user has selected the item.
        Then the item is displayed in the item_display deck and the text field
        is cleared.

        Args:
            obj (Model):
        Returns:
            str: The object as string
        """
        return escape(force_text(obj))

    def format_match(self, obj):
        """
        (HTML) Format item for displaying in the dropdown.

        Args:
            obj (Model):
        Returns:
            str: formatted string, may contain HTML.
        """
        return escape(force_text(obj))

    def format_item_display(self, obj):
        """
        (HTML) format item for displaying item in the selected deck area.

        Args:
            obj (Model):
        Returns:
            str: formatted string, may contain HTML.
        """
        return escape(force_text(obj))

    def get_objects(self, ids):
        """
        This is used to retrieve the currently selected objects for either ManyToMany or ForeignKey.

        Args:
            ids (list): list of primary keys
        Returns:
            list: list of Model objects
        """
        # Inherited models have a OneToOneField (rather than eg AutoField)
        if getattr(self.model._meta.pk, "remote_field", False):
            # Use the type of the field being referenced (2.0+)
            pk_type = self.model._meta.pk.remote_field.field.to_python
        elif getattr(self.model._meta.pk, "rel", False):
            # Use the type of the field being referenced
            pk_type = self.model._meta.pk.rel.field.to_python
        else:
            pk_type = self.model._meta.pk.to_python

        # Return objects in the same order as passed in here
        ids = [pk_type(pk) for pk in ids]
        things = self.model.objects.in_bulk(ids)
        return [things[aid] for aid in ids if aid in things]

    def can_add(self, user, other_model):
        """
        Check if the user has permission to add a ForeignKey or M2M model.

        This enables the green popup + on the widget.
        Default implentation is the standard django permission check.

        Args:
            user (User)
            other_model (Model): the ForeignKey or M2M model to check if the User can add.
        Returns:
            bool
        """
        from django.contrib.contenttypes.models import ContentType
        ctype = ContentType.objects.get_for_model(other_model)
        return user.has_perm("%s.add_%s" % (ctype.app_label, ctype.model))

    def check_auth(self, request):
        """
        By default only request.user.is_staff have access.

        This ensures that nobody can get your data by simply knowing the lookup URL.

        This is called from the ajax_lookup view.

        Public facing forms (outside of the Admin) should implement this to
        allow non-staff to use this LookupChannel.

        Args:
            request (Request)
        Raises:
            PermissionDenied
        """
        if not request.user.is_staff:
            raise PermissionDenied



"""
Test render and submit from the highest Django API level
so we are testing with exactly what Django gives.

Specific errors that are discovered through these tests
should be unit tested in test_fields.py
"""
from __future__ import unicode_literals
import django
from django.forms.models import ModelForm
from django.test import TestCase, Client
from django.contrib.auth.models import User

from tests.models import Book, Author, Person
from ajax_select import fields
try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse

# Other versions will autoload
if django.VERSION[1] < 7:
    from tests import lookups  # noqa

# ---------------  setup ----------------------------------- #


class BookForm(ModelForm):

    class Meta:
        model = Book
        fields = ['name', 'author', 'mentions_persons']

    name = fields.AutoCompleteField('name')
    author = fields.AutoCompleteSelectField('author')
    mentions_persons = fields.AutoCompleteSelectMultipleField('person')


# ---------------  tests ----------------------------------- #

class TestBookForm(TestCase):

    def test_render_no_data(self):
        form = BookForm()
        out = form.as_p()
        # print(out)
        self.assertTrue('autocomplete' in out)
        self.assertTrue('autocompleteselect' in out)
        self.assertTrue('autocompleteselectmultiple' in out)

    def _make_instance(self):
        author = Author.objects.create(name="author")
        book = Book.objects.create(name="book", author=author)
        book.mentions_persons.add(Person.objects.create(name='person'))
        return book

    def _book_data(self, book):
        persons_pks = [person.pk for person in book.mentions_persons.all()]
        mentions_persons = fields.pack_ids(persons_pks)

        return {
            'author': str(book.author.pk),
            'name': book.name,
            'mentions_persons': mentions_persons
        }

    def test_render_instance(self):
        book = self._make_instance()
        form = BookForm(instance=book)
        out = form.as_p()
        # print(out)
        self.assertTrue('autocomplete' in out)
        self.assertTrue('autocompleteselect' in out)
        self.assertTrue('autocompleteselectmultiple' in out)

    def test_render_with_data(self):
        """
        Rendering a form with data already in it
        because it is pre-filled or had errors and is redisplaying.
        """
        book = self._make_instance()
        form = BookForm(data=self._book_data(book))
        out = form.as_p()
        # print(out)
        # should have the values in there somewhere
        self.assertTrue('autocomplete' in out)
        self.assertTrue('autocompleteselect' in out)
        self.assertTrue('autocompleteselectmultiple' in out)

    def test_render_with_initial(self):
        book = self._make_instance()
        # this is data for the form submit
        data = self._book_data(book)
        # initial wants the pks
        data['mentions_persons'] = [p.pk for p in book.mentions_persons.all()]
        form = BookForm(initial=data)
        out = form.as_p()
        # print(out)
        # should have the values in there somewhere
        self.assertTrue('autocomplete' in out)
        self.assertTrue('autocompleteselect' in out)
        self.assertTrue('autocompleteselectmultiple' in out)

    def test_is_valid(self):
        book = self._make_instance()
        form = BookForm(data=self._book_data(book))
        self.assertTrue(form.is_valid())

    def test_full_clean(self):
        book = self._make_instance()
        form = BookForm(data=self._book_data(book))
        form.full_clean()
        data = form.cleaned_data
        # {u'author': <Author: Author object>, u'name': u'book', u'mentions_persons': [u'1']}
        self.assertEqual(data['author'], book.author)
        self.assertEqual(data['name'], book.name)
        # why aren't they instances ?
        self.assertEqual(data['mentions_persons'], [str(p.pk) for p in book.mentions_persons.all()])

    def test_save(self):
        book = self._make_instance()
        form = BookForm(data=self._book_data(book))
        saved = form.save()
        self.assertTrue(saved.pk is not None)

    # def test_save_instance(self):
    #     book = self._make_instance()
    #     form = BookForm(instance=book)
    #     import pdb; pdb.set_trace()
    #     if form.is_valid():
    #         saved = form.save()
    #     else:
    #         print(form.errors)
    #         saved = None
    #     self.assertTrue(saved is not None)
    #     self.assertEqual(saved.pk, book.pk)


class TestAdmin(TestCase):

    def setUp(self):
        self.user = User.objects.create_superuser('admin', 'admin@example.com', 'password')
        self.client = Client()
        ok = self.client.login(username='admin', password='password')
        if not ok:
            raise Exception("Failed to log in")


class TestBookAdmin(TestAdmin):

    """
    Test the admins in tests/admin.py
    """

    def test_get_blank(self):
        app_label = 'tests'
        model = 'book'
        response = self.client.get(reverse('admin:%s_%s_add' % (app_label, model)))
        content = str(response.content)
        # print(content)

        self.assertEqual(response.status_code, 200)

        self.assertTrue('/static/ajax_select/js/ajax_select.js' in content)
        self.assertTrue('autocompleteselectmultiple' in content)
        self.assertTrue('autocompleteselect' in content)
        self.assertTrue('autocomplete' in content)
        self.assertTrue('/admin/tests/author/add/?_popup=1' in content)
        self.assertTrue('/admin/tests/person/add/?_popup=1' in content)


class TestAuthorAdmin(TestAdmin):

    """
    Test an admin with inlines
    """

    def test_get_blank(self):
        app_label = 'tests'
        model = 'author'
        response = self.client.get(reverse('admin:%s_%s_add' % (app_label, model)))
        content = str(response.content)
        # print(content)

        self.assertEqual(response.status_code, 200)

        self.assertTrue('book_set-1-mentions_persons' in content)



from django.apps import apps
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils.module_loading import autodiscover_modules


class LookupChannelRegistry(object):

    """
    Registry for LookupChannels activated for your django project.

    This includes any installed apps that contain lookup.py modules (django 1.7+)
    and any lookups that are explicitly declared in `settings.AJAX_LOOKUP_CHANNELS`
    """
    _registry = {}

    def load_channels(self):
        """
        Called when loading the application. Cannot be called a second time,
        (eg. for testing) as Django will not re-import and re-register anything.
        """
        autodiscover_modules('lookups')

        if hasattr(settings, 'AJAX_LOOKUP_CHANNELS'):
            self.register(settings.AJAX_LOOKUP_CHANNELS)

    def register(self, lookup_specs):
        """Register a set of lookup definitions.

        Args:
            lookup_specs (dict): One or more LookupChannel specifications
                - `{'channel': LookupChannelSubclass}`
                - `{'channel': ('module.of.lookups', 'MyLookupClass')}`
                - `{'channel': {'model': 'MyModelToBeLookedUp', 'search_field': 'field_to_search'}}`
        """
        for channel, spec in lookup_specs.items():
            if spec is None:  # unset
                if channel in self._registry:
                    del self._registry[channel]
            else:
                self._registry[channel] = spec

    def get(self, channel):
        """Find the LookupChannel class for the named channel and instantiate it.

        Args:
            channel (string):  - name that the lookup channel was registered at
        Returns:
            LookupChannel
        Raises:
            ImproperlyConfigured - if channel is not found.
            Exception - invalid lookup_spec was stored in registery
        """
        from ajax_select import LookupChannel

        try:
            lookup_spec = self._registry[channel]
        except KeyError:
            raise ImproperlyConfigured(
                "No ajax_select LookupChannel named %(channel)r is registered." % {'channel': channel})

        if (type(lookup_spec) is type) and issubclass(lookup_spec, LookupChannel):
            return lookup_spec()
            # damnit python.
            # ideally this would match regardless of how you imported the parent class
            # but these are different classes:
            # from ajax_select.lookup_channel import LookupChannel
            # from ajax_select import LookupChannel
        elif isinstance(lookup_spec, dict):
            # 'channel' : dict(model='app.model', search_field='title' )
            #  generate a simple channel dynamically
            return self.make_channel(lookup_spec['model'], lookup_spec['search_field'])
        elif isinstance(lookup_spec, tuple):
            # a tuple
            # 'channel' : ('app.module','LookupClass')
            #  from app.module load LookupClass and instantiate
            lookup_module = __import__(lookup_spec[0], {}, {}, [''])
            lookup_class = getattr(lookup_module, lookup_spec[1])
            return lookup_class()
        else:
            raise Exception("Invalid lookup spec: %s" % lookup_spec)

    def is_registered(self, channel):
        return channel in self._registry

    def make_channel(self, app_model, arg_search_field):
        """Automatically make a LookupChannel.

        Args:
            app_model (str):   app_name.ModelName
            arg_search_field (str):  the field to search against and to display in search results
        Returns:
            LookupChannel
        """
        from ajax_select import LookupChannel
        app_label, model_name = app_model.split(".")

        class MadeLookupChannel(LookupChannel):

            model = get_model(app_label, model_name)
            search_field = arg_search_field

        return MadeLookupChannel()


registry = LookupChannelRegistry()


def get_model(app_label, model_name):
    """Loads the model given an 'app_label' 'ModelName'"""
    return apps.get_model(app_label, model_name)


def register(channel):
    """Decorator to register a LookupClass.

    Example::
        from ajax_select import LookupChannel, register

        @register('agent')
        class AgentLookup(LookupClass):

            def get_query(self):
                ...
            def format_item(self):
                ...

    """

    def _wrapper(lookup_class):
        if not channel:
            raise ValueError('Lookup Channel must have a channel name')

        registry.register({channel: lookup_class})

        return lookup_class

    return _wrapper



"""JQuery-Ajax Autocomplete fields for Django Forms."""
__version__ = "1.7.0"
__author__ = "crucialfelix"
__contact__ = "crucialfelix@gmail.com"
__homepage__ = "https://github.com/crucialfelix/django-ajax-selects/"

from ajax_select.registry import registry, register  # noqa
from ajax_select.helpers import make_ajax_form, make_ajax_field  # noqa
from ajax_select.lookup_channel import LookupChannel  # noqa

# django 1.7+ will use the new AppConfig api
# It will load all your lookups.py modules
# and any specified in settings.AJAX_LOOKUP_CHANNELS
# It will do this after all apps are imported.
from django.apps import AppConfig  # noqa
default_app_config = 'ajax_select.apps.AjaxSelectConfig'



"""Testing if lookups that are not in a file named lookups.py can be loaded correctly."""

import ajax_select
from tests.models import Book


class BookLookup(ajax_select.LookupChannel):

    model = Book



DEBUG = True
USE_TZ = True
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
    }
}
ROOT_URLCONF = "tests.urls"
INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sites",
    'django.contrib.messages',
    'django.contrib.sessions',
    'django.contrib.admin',
    'django.contrib.staticfiles',
    "ajax_select",
    "tests"
]
SITE_ID = 1
MIDDLEWARE = MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware'
)
STATIC_URL = '/static/'
SECRET_KEY = 'inyd5fc5pymlsv@hwoc5+3_6*cm0erlxzv6i-wl0jm_kt-6rp9'

AJAX_LOOKUP_CHANNELS = {
    # tuple points to a module and class to load
    'book': ('tests.other_lookups', 'BookLookup'),
    # dict specifies an automatically constructed LookupChannel
    'author': {'model': 'tests.Author', 'search_field': 'name'},
    # unset a channel that a third-party app specified
    'user': None,
    'was-never-a-channel': None
    # LookupChannels in lookups.py are auto-loaded
}

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            # insert your TEMPLATE_DIRS here
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                # Insert your TEMPLATE_CONTEXT_PROCESSORS here or use this
                # list if you haven't customized them:
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]




from django.contrib.auth.models import User
from django.test import TestCase

from tests.lookups import PersonWithTitleLookup, UserLookup
from tests.models import PersonWithTitle


class TestLookups(TestCase):

    def test_get_objects(self):
        user1 = User.objects.create(username='user1',
                                    email='user1@example.com',
                                    password='password')
        user2 = User.objects.create(username='user2',
                                    email='user2@example.com',
                                    password='password')
        lookup = UserLookup()
        # deliberately asking for them backwards:
        users = lookup.get_objects([user2.id, user1.id])
        self.assertEqual(len(users), 2)
        # to make sure they come back in the order requested
        u2, u1 = users
        self.assertEqual(u1, user1)
        self.assertEqual(u2, user2)

    def test_get_objects_inherited_model(self):
        """
        Tests that get_objects works with inherited models
        """
        one = PersonWithTitle.objects.create(name='one', title='The One')
        two = PersonWithTitle.objects.create(name='two', title='The Other')
        lookup = PersonWithTitleLookup()
        users = lookup.get_objects([one.id, two.id])
        self.assertEqual(len(users), 2)
        u1, u2 = users
        self.assertEqual(u1, one)
        self.assertEqual(u2, two)



from __future__ import unicode_literals

import json

from django import forms
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.db.models import Model
from django.forms.utils import flatatt
from django.template.defaultfilters import force_escape
from django.template.loader import render_to_string
from django.utils.encoding import force_text
from django.utils.safestring import mark_safe
from django.utils.six import text_type
from django.utils.translation import ugettext as _
from django.utils.module_loading import import_string

from ajax_select.registry import registry

try:
    from django.urls import reverse
except ImportError:
    # < django 1.10
    from django.core.urlresolvers import reverse


as_default_help = 'Enter text to search.'


def _media(self):
    # unless AJAX_SELECT_BOOTSTRAP == False
    # then load jquery and jquery ui + default css
    # where needed
    js = ('ajax_select/js/bootstrap.js', 'ajax_select/js/ajax_select.js')
    try:
        if not settings.AJAX_SELECT_BOOTSTRAP:
            js = ('ajax_select/js/ajax_select.js',)
    except AttributeError:
        pass
    return forms.Media(css={'all': ('ajax_select/css/ajax_select.css',)}, js=js)


json_encoder = import_string(getattr(settings, 'AJAX_SELECT_JSON_ENCODER',
                                     'django.core.serializers.json.DjangoJSONEncoder'))


###############################################################################


class AutoCompleteSelectWidget(forms.widgets.TextInput):

    """
    Widget to search for a model and return it as text for use in a CharField.
    """

    media = property(_media)

    add_link = None

    def __init__(self,
                 channel,
                 help_text='',
                 show_help_text=True,
                 plugin_options=None,
                 *args,
                 **kwargs):
        self.plugin_options = plugin_options or {}
        super(forms.widgets.TextInput, self).__init__(*args, **kwargs)
        self.channel = channel
        self.help_text = help_text
        self.show_help_text = show_help_text

    def render(self, name, value, attrs=None):
        value = value or ''

        final_attrs = self.build_attrs(self.attrs)
        final_attrs.update(attrs or {})
        final_attrs.pop('required', None)
        self.html_id = final_attrs.pop('id', name)

        current_repr = ''
        initial = None
        lookup = registry.get(self.channel)
        if value:
            objs = lookup.get_objects([value])
            try:
                obj = objs[0]
            except IndexError:
                raise Exception("%s cannot find object:%s" % (lookup, value))
            current_repr = lookup.format_item_display(obj)
            initial = [current_repr, obj.pk]

        if self.show_help_text:
            help_text = self.help_text
        else:
            help_text = ''

        context = {
            'name': name,
            'html_id': self.html_id,
            'current_id': value,
            'current_repr': current_repr,
            'help_text': help_text,
            'extra_attrs': mark_safe(flatatt(final_attrs)),
            'func_slug': self.html_id.replace("-", ""),
            'add_link': self.add_link,
        }
        context.update(
            make_plugin_options(lookup, self.channel, self.plugin_options, initial))
        templates = (
            'ajax_select/autocompleteselect_%s.html' % self.channel,
            'ajax_select/autocompleteselect.html')
        out = render_to_string(templates, context)
        return mark_safe(out)

    def value_from_datadict(self, data, files, name):
        return data.get(name, None)

    def id_for_label(self, id_):
        return '%s_text' % id_


class AutoCompleteSelectField(forms.fields.CharField):

    """Form field to select a Model for a ForeignKey db field."""

    channel = None

    def __init__(self, channel, *args, **kwargs):
        self.channel = channel

        widget_kwargs = dict(
            channel=channel,
            help_text=kwargs.get('help_text', _(as_default_help)),
            show_help_text=kwargs.pop('show_help_text', True),
            plugin_options=kwargs.pop('plugin_options', {})
        )
        widget_kwargs.update(kwargs.pop('widget_options', {}))
        kwargs["widget"] = AutoCompleteSelectWidget(**widget_kwargs)
        super(AutoCompleteSelectField, self).__init__(
            max_length=255, *args, **kwargs)

    def clean(self, value):
        if value:
            lookup = registry.get(self.channel)
            objs = lookup.get_objects([value])
            if len(objs) != 1:
                # someone else might have deleted it while you were editing
                # or your channel is faulty
                # out of the scope of this field to do anything more than
                # tell you it doesn't exist
                raise forms.ValidationError(
                    "%s cannot find object: %s" % (lookup, value))
            return objs[0]
        else:
            if self.required:
                raise forms.ValidationError(self.error_messages['required'])
            return None

    def check_can_add(self, user, model):
        _check_can_add(self, user, model)

    def has_changed(self, initial, data):
        # 1 vs u'1'
        initial_value = initial if initial is not None else ''
        data_value = data if data is not None else ''
        return text_type(initial_value) != text_type(data_value)


###############################################################################


class AutoCompleteSelectMultipleWidget(forms.widgets.SelectMultiple):

    """
    Widget to select multiple models for a ManyToMany db field.
    """

    media = property(_media)

    add_link = None

    def __init__(self,
                 channel,
                 help_text='',
                 show_help_text=True,
                 plugin_options=None,
                 *args,
                 **kwargs):
        super(AutoCompleteSelectMultipleWidget, self).__init__(*args, **kwargs)
        self.channel = channel

        self.help_text = help_text
        self.show_help_text = show_help_text
        self.plugin_options = plugin_options or {}

    def render(self, name, value, attrs=None):

        if value is None:
            value = []

        final_attrs = self.build_attrs(self.attrs)
        final_attrs.update(attrs or {})
        final_attrs.pop('required', None)
        self.html_id = final_attrs.pop('id', name)

        lookup = registry.get(self.channel)

        values = list(value)
        if all([isinstance(v, Model) for v in values]):
            objects = values
        else:
            objects = lookup.get_objects(values)

        current_ids = pack_ids([obj.pk for obj in objects])

        # text repr of currently selected items
        initial = [
            [lookup.format_item_display(obj), obj.pk]
            for obj in objects
        ]

        if self.show_help_text:
            help_text = self.help_text
        else:
            help_text = ''

        context = {
            'name': name,
            'html_id': self.html_id,
            'current': value,
            'current_ids': current_ids,
            'current_reprs': mark_safe(
                json.dumps(initial, cls=json_encoder)
            ),
            'help_text': help_text,
            'extra_attrs': mark_safe(flatatt(final_attrs)),
            'func_slug': self.html_id.replace("-", ""),
            'add_link': self.add_link,
        }
        context.update(
            make_plugin_options(lookup, self.channel, self.plugin_options, initial))
        templates = ('ajax_select/autocompleteselectmultiple_%s.html' % self.channel,
                     'ajax_select/autocompleteselectmultiple.html')
        out = render_to_string(templates, context)
        return mark_safe(out)

    def value_from_datadict(self, data, files, name):
        # eg. 'members': ['|229|4688|190|']
        return [val for val in data.get(name, '').split('|') if val]

    def id_for_label(self, id_):
        return '%s_text' % id_


class AutoCompleteSelectMultipleField(forms.fields.CharField):

    """
    Form field to select multiple models for a ManyToMany db field.
    """

    channel = None

    def __init__(self, channel, *args, **kwargs):
        self.channel = channel

        help_text = kwargs.get('help_text')
        show_help_text = kwargs.pop('show_help_text', False)

        if not (help_text is None):
            # '' will cause translation to fail
            # should be ''
            if isinstance(help_text, str):
                help_text = force_text(help_text)
            # django admin appends "Hold down "Control",..." to the help text
            # regardless of which widget is used. so even when you specify an
            # explicit help text it appends this other default text onto the end.
            # This monkey patches the help text to remove that
            if help_text != '':
                if not isinstance(help_text, text_type):
                    # ideally this could check request.LANGUAGE_CODE
                    translated = help_text.translate(settings.LANGUAGE_CODE)
                else:
                    translated = help_text
                dh = 'Hold down "Control", or "Command" on a Mac, to select more than one.'
                django_default_help = _(dh).translate(settings.LANGUAGE_CODE)
                if django_default_help in translated:
                    cleaned_help = translated.replace(
                        django_default_help, '').strip()
                    # probably will not show up in translations
                    if cleaned_help:
                        help_text = cleaned_help
                    else:
                        help_text = ""
                        show_help_text = False
        else:
            help_text = _(as_default_help)

        # django admin will also show help text outside of the display
        # area of the widget.  this results in duplicated help.
        # it should just let the widget do the rendering
        # so by default do not show it in widget
        # if using in a normal form then set to True when creating the field
        widget_kwargs = {
            'channel': channel,
            'help_text': help_text,
            'show_help_text': show_help_text,
            'plugin_options': kwargs.pop('plugin_options', {})
        }
        widget_kwargs.update(kwargs.pop('widget_options', {}))
        kwargs['widget'] = AutoCompleteSelectMultipleWidget(**widget_kwargs)
        kwargs['help_text'] = help_text

        super(AutoCompleteSelectMultipleField, self).__init__(*args, **kwargs)

    def clean(self, value):
        if not value and self.required:
            raise forms.ValidationError(self.error_messages['required'])
        return value  # a list of primary keys from widget value_from_datadict

    def check_can_add(self, user, model):
        _check_can_add(self, user, model)

    def has_changed(self, initial_value, data_value):
        # [1, 2] vs [u'1', u'2']
        ivs = [text_type(v) for v in (initial_value or [])]
        dvs = [text_type(v) for v in (data_value or [])]
        return ivs != dvs

###############################################################################


class AutoCompleteWidget(forms.TextInput):

    """
    Widget to select a search result and enter the result as raw text in the
    text input field. The user may also simply enter text and ignore any
    auto complete suggestions.
    """

    media = property(_media)

    channel = None
    help_text = ''
    html_id = ''

    def __init__(self, channel, *args, **kwargs):
        self.channel = channel
        self.help_text = kwargs.pop('help_text', '')
        self.show_help_text = kwargs.pop('show_help_text', True)
        self.plugin_options = kwargs.pop('plugin_options', {})

        super(AutoCompleteWidget, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None):

        initial = value or ''
        final_attrs = self.build_attrs(self.attrs)
        final_attrs.update(attrs or {})
        self.html_id = final_attrs.pop('id', name)
        final_attrs.pop('required', None)

        lookup = registry.get(self.channel)
        if self.show_help_text:
            help_text = self.help_text
        else:
            help_text = ''

        context = {
            'current_repr': initial,
            'current_id': initial,
            'help_text': help_text,
            'html_id': self.html_id,
            'name': name,
            'extra_attrs': mark_safe(flatatt(final_attrs)),
            'func_slug': self.html_id.replace("-", ""),
        }
        context.update(
            make_plugin_options(lookup, self.channel, self.plugin_options, initial))
        templates = ('ajax_select/autocomplete_%s.html' % self.channel,
                     'ajax_select/autocomplete.html')
        return mark_safe(render_to_string(templates, context))


class AutoCompleteField(forms.CharField):

    """
    A CharField that uses an AutoCompleteWidget to lookup matching
    and stores the result as plain text.
    """
    channel = None

    def __init__(self, channel, *args, **kwargs):
        self.channel = channel

        widget_kwargs = dict(
            help_text=kwargs.get('help_text', _(as_default_help)),
            show_help_text=kwargs.pop('show_help_text', True),
            plugin_options=kwargs.pop('plugin_options', {})
        )
        widget_kwargs.update(kwargs.pop('widget_options', {}))
        if 'attrs' in kwargs:
            widget_kwargs['attrs'] = kwargs.pop('attrs')
        widget = AutoCompleteWidget(channel, **widget_kwargs)

        defaults = {'max_length': 255, 'widget': widget}
        defaults.update(kwargs)

        super(AutoCompleteField, self).__init__(*args, **defaults)


###############################################################################

def _check_can_add(self, user, related_model):
    """
    Check if the User can create a related_model.

    If the LookupChannel implements check_can_add() then use this.

    Else uses Django's default permission system.

    If it can add, then enable the widget to show the green + link
    """
    lookup = registry.get(self.channel)
    if hasattr(lookup, 'can_add'):
        can_add = lookup.can_add(user, related_model)
    else:
        ctype = ContentType.objects.get_for_model(related_model)
        can_add = user.has_perm("%s.add_%s" % (ctype.app_label, ctype.model))
    if can_add:
        app_label = related_model._meta.app_label
        model = related_model._meta.object_name.lower()
        self.widget.add_link = reverse(
            'admin:%s_%s_add' % (app_label, model)) + '?_popup=1'


def autoselect_fields_check_can_add(form, model, user):
    """
    Check the form's fields for any autoselect fields and enable their
    widgets with green + button if permissions allow then to create the
    related_model.
    """
    for name, form_field in form.declared_fields.items():
        if isinstance(form_field, (AutoCompleteSelectMultipleField, AutoCompleteSelectField)):
            db_field = model._meta.get_field(name)
            if hasattr(db_field, "remote_field"):
                form_field.check_can_add(user, db_field.remote_field.model)
            else:
                form_field.check_can_add(user, db_field.rel.to)


def make_plugin_options(lookup, channel_name, widget_plugin_options, initial):
    """ Make a JSON dumped dict of all options for the jQuery ui plugin."""
    po = {}
    if initial:
        po['initial'] = initial
    po.update(getattr(lookup, 'plugin_options', {}))
    po.update(widget_plugin_options)
    if not po.get('source'):
        po['source'] = reverse('ajax_lookup', kwargs={'channel': channel_name})

    # allow html unless explicitly set
    if po.get('html') is None:
        po['html'] = True

    return {
        'plugin_options': mark_safe(json.dumps(po, cls=json_encoder)),
        'data_plugin_options': force_escape(
            json.dumps(po, cls=json_encoder)
        )
    }


def pack_ids(ids):
    if ids:
        # |pk|pk| of current
        return "|" + "|".join(str(pk) for pk in ids) + "|"
    else:
        return "|"



import json
from django.http import HttpResponse
from django.utils.encoding import force_text
from ajax_select import registry


def ajax_lookup(request, channel):

    """Load the named lookup channel and lookup matching models.

    GET or POST should contain 'term'

    Returns:
        HttpResponse - JSON: `[{pk: value: match: repr:}, ...]`
    Raises:
        PermissionDenied - depending on the LookupChannel's implementation of check_auth
    """

    # it should come in as GET unless global $.ajaxSetup({type:"POST"}) has been set
    # in which case we'll support POST
    if request.method == "GET":
        # we could also insist on an ajax request
        if 'term' not in request.GET:
            return HttpResponse('')
        query = request.GET['term']
    else:
        if 'term' not in request.POST:
            return HttpResponse('')  # suspicious
        query = request.POST['term']

    lookup = registry.get(channel)
    if hasattr(lookup, 'check_auth'):
        lookup.check_auth(request)

    if len(query) >= getattr(lookup, 'min_length', 1):
        instances = lookup.get_query(query, request)
    else:
        instances = []

    results = json.dumps([
        {
            'pk': force_text(getattr(item, 'pk', None)),
            'value': lookup.get_result(item),
            'match': lookup.format_match(item),
            'repr': lookup.format_item_display(item)
        } for item in instances
    ])

    response = HttpResponse(results, content_type='application/json')
    response['Cache-Control'] = 'max-age=0, must-revalidate, no-store, no-cache;'
    return response




from django.contrib import admin
from ajax_select.admin import AjaxSelectAdmin, AjaxSelectAdminTabularInline
from tests.models import Author, Book, Person
from tests.test_integration import BookForm


@admin.register(Book)
class BookAdmin(AjaxSelectAdmin):
    form = BookForm


class BookInline(AjaxSelectAdminTabularInline):

    model = Book
    form = BookForm
    extra = 2


@admin.register(Author)
class AuthorAdmin(AjaxSelectAdmin):

    inlines = [
        BookInline
    ]


@admin.register(Person)
class PersonAdmin(admin.ModelAdmin):
    pass




from django.conf.urls import url, include
from django.conf.urls.static import static
from django.contrib import admin
from django.conf import settings
from ajax_select import urls as ajax_select_urls


admin.autodiscover()

urlpatterns = [
    url(r'^ajax_lookups/', include(ajax_select_urls)),
    url(r'^admin/', admin.site.urls),
] + static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)



#!/usr/bin/env python

from setuptools import setup

setup(
    name='django-ajax-selects',
    version='1.7.0',
    description='Edit ForeignKey, ManyToManyField and CharField in Django Admin using jQuery UI AutoComplete.',
    author='Chris Sattinger',
    author_email='crucialfelix@gmail.com',
    url='https://github.com/crucialfelix/django-ajax-selects/',
    packages=['ajax_select'],
    package_data={
        'ajax_select':
        [
            '*.py',
            '*.txt',
            '*.md',
            'static/ajax_select/css/*',
            'static/ajax_select/images/*',
            'static/ajax_select/js/*',
            'templates/ajax_select/*.html'
        ]
    },
    include_package_data=True,
    zip_safe=False,
    license="MIT",
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Development Status :: 5 - Production/Stable",
        'Environment :: Web Environment',
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: User Interfaces",
        "Framework :: Django",
    ],
    long_description="""\
Edit ForeignKey, ManyToManyField and CharField in Django Admin using jQuery UI AutoComplete.

- Customize search query
- Query other resources besides Django ORM
- Format results with HTML
- Customize styling
- Customize security policy
- Add additional custom UI alongside widget
- Integrate with other UI elements elsewhere on the page using the javascript API
- Works in Admin as well as in normal views

- Django >=1.8, <=2.1
- Python >=2.7, <=3.7
"""
)



from django.db.models.fields.related import ForeignKey, ManyToManyField
from django.forms.models import ModelForm
from django.utils.encoding import force_text
from django.utils.text import capfirst
from django.utils.translation import ugettext_lazy as _


def make_ajax_form(model, fieldlist, superclass=ModelForm, show_help_text=False, **kwargs):
    """Creates a ModelForm subclass with AutoComplete fields.

    Args:
        model (type): Model class for which you are making the ModelForm
        fieldlist (dict): {field_name -> channel_name, ...}
        superclass (type): optional ModelForm superclass
        show_help_text (bool): suppress or show the widget help text

    Returns:
        ModelForm: a ModelForm suitable for use in an Admin

    Usage::

        from django.contrib import admin
        from ajax_select import make_ajax_form
        from yourapp.models import YourModel

        @admin.register(YourModel)
        class YourModelAdmin(Admin):

            form = make_ajax_form(YourModel, {
                'contacts': 'contact',  # ManyToManyField
                'author':'contact'      # ForeignKeyField
            })

    Where 'contacts' is a ManyToManyField specifying to use the lookup channel 'contact'
    and 'author' is a ForeignKeyField specifying here to also use the same lookup channel 'contact'

    """
    # will support previous arg name for several versions before deprecating
    # TODO: time to go
    if 'show_m2m_help' in kwargs:
        show_help_text = kwargs.pop('show_m2m_help')

    class TheForm(superclass):

        class Meta:
            exclude = []

        setattr(Meta, 'model', model)
        if hasattr(superclass, 'Meta'):
            if hasattr(superclass.Meta, 'fields'):
                setattr(Meta, 'fields', superclass.Meta.fields)
            if hasattr(superclass.Meta, 'exclude'):
                setattr(Meta, 'exclude', superclass.Meta.exclude)
            if hasattr(superclass.Meta, 'widgets'):
                setattr(Meta, 'widgets', superclass.Meta.widgets)

    for model_fieldname, channel in fieldlist.items():
        f = make_ajax_field(model, model_fieldname, channel, show_help_text)

        TheForm.declared_fields[model_fieldname] = f
        TheForm.base_fields[model_fieldname] = f

    return TheForm


def make_ajax_field(related_model, fieldname_on_model, channel, show_help_text=False, **kwargs):
    """Makes an AutoComplete field for use in a Form.

    Args:
        related_model (Model): model of the related object
        fieldname_on_model (str): field name on the model being edited
        channel (str): channel name of a registered LookupChannel
        show_help_text (bool): show or supress help text below the widget
            Django admin will show help text below the widget, but not for ManyToMany inside of admin inlines
            This setting will show the help text inside the widget itself.
        kwargs: optional args

            - help_text: default is the model db field's help_text.
                            None will disable all help text
            - label: default is the model db field's verbose name
            - required: default is the model db field's (not) blank

    Returns:
        (AutoCompleteField, AutoCompleteSelectField, AutoCompleteSelectMultipleField): field
    """
    from ajax_select.fields import AutoCompleteField, \
        AutoCompleteSelectMultipleField, \
        AutoCompleteSelectField

    field = related_model._meta.get_field(fieldname_on_model)
    if 'label' not in kwargs:
        kwargs['label'] = _(capfirst(force_text(field.verbose_name)))

    if ('help_text' not in kwargs) and field.help_text:
        kwargs['help_text'] = field.help_text
    if 'required' not in kwargs:
        kwargs['required'] = not field.blank

    kwargs['show_help_text'] = show_help_text
    if isinstance(field, ManyToManyField):
        f = AutoCompleteSelectMultipleField(
            channel,
            **kwargs)
    elif isinstance(field, ForeignKey):
        f = AutoCompleteSelectField(
            channel,
            **kwargs)
    else:
        f = AutoCompleteField(
            channel,
            **kwargs)
    return f




from django.db import models


class Person(models.Model):

    name = models.CharField(max_length=50)
    email = models.EmailField(null=True, blank=True)

    class Meta:
        app_label = 'tests'


class PersonWithTitle(Person):

    """
    Testing an inherited model (multi-table)
    """

    title = models.CharField(max_length=150)

    class Meta:
        app_label = 'tests'


class Author(models.Model):

    name = models.CharField(max_length=50)

    class Meta:
        app_label = 'tests'


class Book(models.Model):

    """ Book has no admin, its an inline in the Author admin"""

    author = models.ForeignKey(Author, null=True, on_delete=models.CASCADE)
    name = models.CharField(max_length=50)
    mentions_persons = models.ManyToManyField(Person, help_text="MENTIONS PERSONS HELP TEXT")

    class Meta:
        app_label = 'tests'
